/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "AssociationRule.h"
#include "FITree.h"

// Standard output function
ostream& operator<<(ostream& os, const AssociationRule& arule) {
	os << *(arule.condition) << "=> " << *(arule.consequent);
	return os;
}

int AssociationRule::CREATIONS = 0;
int AssociationRule::DELETIONS = 0;

/*
bool ConditionComparator::operator()(const AssociationRule* ar1, const AssociationRule* ar2) const {
	return (ar1->condition < ar2->condition);
}

bool ConsequentComparator::operator()(const AssociationRule* ar1, const AssociationRule* ar2) const {
	return (ar1->consequent < ar2->consequent);
}
*/

AssociationRule::AssociationRule() {
	AssociationRule::CREATIONS++;

	itemset = NULL;
	condition = NULL;
	consequent = NULL;
}

AssociationRule::AssociationRule(FITree* i, FITree* cond, FITree* cons) {
	AssociationRule::CREATIONS++;

	itemset = i;
	condition = cond;
	consequent = cons;
}

AssociationRule::AssociationRule(const AssociationRule& arule) {
	AssociationRule::CREATIONS++;

	itemset = arule.itemset;
	condition = arule.condition;
	consequent = arule.consequent;
}

AssociationRule::~AssociationRule() {
	AssociationRule::DELETIONS++;
}

// Stores the association rule to the FITree for further use (i.e. incremental pruning)
void AssociationRule::store() {
	condition->consequent_rules[consequent] = this;
	consequent->condition_rules[condition] = this;
}

// Gets all stored condition subrules
vector<AssociationRule*>* AssociationRule::conditionSubrules() {
	vector<AssociationRule*>* subrules = new vector<AssociationRule*>();
	
	vector<FITree*>* condition_subsets = condition->subsets();
	
	AssociationRule* subrule;
	
	for (int i = 0; i < condition_subsets->size(); i++) {
		subrule = consequent->condition_rules[(*condition_subsets)[i]];
		if (subrule != NULL) {
			subrules->push_back(subrule);
		}
	}
	
#ifndef _STORE_SUBSETS
	delete condition_subsets;
#endif	
	return subrules;
}

// Gets all stored consequent subrules
vector<AssociationRule*>* AssociationRule::consequentSubrules() {
	vector<AssociationRule*>* subrules = new vector<AssociationRule*>();
	
	vector<FITree*>* consequent_subsets = consequent->subsets();
	
	AssociationRule* subrule;
	
	for (int i = 0; i < consequent_subsets->size(); i++) {
		subrule = condition->consequent_rules[(*consequent_subsets)[i]];
		if (subrule != NULL) {
			subrules->push_back(subrule);
		}
	}
	
#ifndef _STORE_SUBSETS
	delete consequent_subsets;
#endif	
	return subrules;
}
