/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_CONFIDENCECONSTRAINT
#define FIRM_CONFIDENCECONSTRAINT

#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for calculating the confidence of an association rule.
 * @author Juho Muhonen
 * @modified 25.10.2005 created
 */
class ConfidenceConstraint: public RuleConstraint {
private:
	/**
	 * The confidence of the last evaluated pattern.
	 */
	double last_confidence;
	
	/**
	 * The minimum confidence used in validating.
	 */
	double min_confidence;
		
public:	
	/**
	 * Create a confidence constraint with minimum confidence of 0
	 */
	ConfidenceConstraint();
	
	/**
	 * Create a confidence constraint with a minimum confidence.
	 */
	ConfidenceConstraint(double);
	
	/**
	 * Set the minimum confidence of this constraint.
	 */
	void setMinConfidence(double d);
	
	/**
	 * Return the confidence of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum confidence.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
