/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_ECLAT
#define FIRM_ECLAT

#include <map>
using std::multimap;

class FITree;
class AVTree;
class ContextTree;

/**
 * Eclat for itemsets
 * @author Juho Muhonen
 * @modification 10.10.2005 commented all
 * @modification 10.10.2005 void generateFrequentClosedContextSets(ContextTree*, int);
 */
class Eclat {
private:
	/**
	 * Usage of diffsets.
	 */
	bool diffsets;
	
	/**
	 * Usage of FITree comression
	 */
	bool expand_all_itemsets;

	/**
	 * Map for storing found closed itemsets
	 */
	multimap<int, FITree*> closed_itemsets;
	
	/**
	 * Checks if an itemset has a closed superset with same support.
	 */
	bool hasClosedSuperset(FITree*);
	
	/**
	 * Removes a closed set from the hashmap
	 */
	void removeClosedSet(FITree*);
	
	/**
	 * Recursively generate all frequent itemsets from attribute-value data
	 */
	void generateFrequentSets(FITree*, int);

	/**
	 * Recursively generate all frequent closed itemsets from attribute-value data
	 */
	void generateFrequentClosedSets(FITree*, int);
	
	/**
	 * Recursively generate all frequent itemsets from context-data
	 */
	void generateFrequentContextSets(ContextTree*, FITree*, int);

	/**
	 * Recursively generate all frequent closed itemsets from context data
	 */
	void generateFrequentClosedContextSets(ContextTree*, FITree*, int);

public:
	/**
	 * Create a new miner.
	 */
	Eclat();
	
	/**
	 * Create a new miner with certain usage options.
	 */
	Eclat(bool, bool);

	/**
	 * The destructor.
	 */
	~Eclat();
	
	/**
	 * Set the usage of diffsets
	 */
	void setDiffsets(bool);
	
	/**
	 * Set the usage of FITree compression
	 */
	void setExpand(bool);
	
	/**
	 * Generate all frequent itemsets from attribute-value data with given support threshold
	 */
	FITree* generateFrequentSets(AVTree*, int);

	/**
	 * Generate all frequent itemsets from context data with given support threshold
	 */
	FITree* generateFrequentSets(ContextTree*, int);

	/**
	 * Generate all frequent closed itemsets from attribute-value data with given support threshold
	 */
	FITree* generateFrequentClosedSets(AVTree*, int);

	/**
	 * Generate all frequent closed itemsets from context data with given support threshold
	 */
	FITree* generateFrequentClosedContextSets(ContextTree*, int);
};
#endif
