/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_JMEASURECONSTRAINT
#define FIRM_JMEASURECONSTRAINT

#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for computing the J-Measure of an association rule.
 * @author Juho Muhonen
 * @modification 11.11.2005 created
 */
class JMeasureConstraint: public RuleConstraint {
private:
	/**
	 * The J-measure of the last evaluated pattern.
	 */
	double last_jmeasure;
	
	/**
	 * The minimum J-measure used in validating.
	 */
	double min_jmeasure;
		
public:	
	/**
	 * Create a J-measure constraint with minimum J-measure of 0
	 */
	JMeasureConstraint();
	
	/**
	 * Create a J-measure constraint with a minimum J-measure.
	 */
	JMeasureConstraint(double);
	
	/**
	 * Set the minimum J-measure of this constraint.
	 */
	void setMinJMeasure(double d);
	
	/**
	 * Return the J-measure of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum J-measure.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
