/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_LIFTCONSTRAINT
#define FIRM_LIFTCONSTRAINT

#include "SetConstraint.h"
#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for calculating the lift of an itemset or an association rule.
 * @author Juho Muhonen
 * @modified 25.10.2005 created
 */
class LiftConstraint:public SetConstraint, public RuleConstraint {
private:
	/**
	 * The lift of the last evaluated pattern.
	 */
	double last_lift;
	
	/**
	 * The minimum lift used in validating.
	 */
	double min_lift;
		
public:	
	/**
	 * Create a lift constraint with minimum lift of 0
	 */
	LiftConstraint();
	
	/**
	 * Create a lift constraint with a minimum lift.
	 */
	LiftConstraint(double);
	
	/**
	 * Set the minimum lift of this constraint.
	 */
	void setMinLift(double d);
	
	/**
	 * Return the lift of an itemset.
	 */
	double value(FITree*);

	/**
	 * Return the lift of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an itemset.
	 */
	bool validate(FITree*);

	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum lift.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;

	/**
	 * Return the highest possible value of this constraint.
	 */
	double upperBound() const;
	
	/**
	 * Return the lowest possible value of this constraint.
	 */
	double lowerBound() const;
	
	/**
	 * Return the "zero point" of this constraint.
	 */
	double zeroLimit() const;

};
#endif
