/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_OUTPUTGENERATOR
#define FIRM_OUTPUTGENERATOR

#include <string>
using std::string;

class AVTree;
class ContextTree;
class SetConstraint;
class RuleConstraint;
class FITree;
class AssociationRule;

/**
 * A Class that is the superclass of all output generators
 * @author Juho Muhonen
 * @modified 6.10.2005 commented all
 */
class OutputGenerator {
public:
	/**
	 * Set the output name. For example the name of the file where the output is going.
	 */
	virtual void setOutputName(string) {};

	/**
	 * Add a set constraint to this output generator. If the generator is asked to output an
	 * itemset, it should first validate the itemset for all the set constraints that have been added.
	 */
	virtual void addSetConstraint(SetConstraint*) {};
	
	/**
	 * Add a rule constraint to this output generator. If the generator is asked to output an
	 * association rule, it should first validate the rule for all the rule constraints that have been added.
	 */
	virtual void addRuleConstraint(RuleConstraint*) {};
	
	/**
	 * Remove a set constraint by constraint name.
	 */
	virtual void removeSetConstraint(string) {};

	/**
	 * Remove a rule constraint by constraint name.
	 */
	virtual void removeRuleConstraint(string) {};
	
	/**
	 * Output an itemset.
	 */
	virtual void outputSet(FITree*) {};

	/**
	 * Output all itemset originating from a given root set.
	 */
	virtual void outputSets(FITree*) {};
	
	/**
	 * Output an association rule.
	 */
	virtual void outputRule(AssociationRule*) {};

	/**
	 * Output all association rules in an itemset tree.
	 */
	virtual void outputRules(FITree*) {};

	/**
	 * A call to finalize is made when the output generator is no longer used. This should close all streams still open.
	 */
	virtual void finalize() {};
};
#endif
