/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __PARAMETER_PARSER
#define __PARAMETER_PARSER

#include <map>
using std::map;

#include <string>
using std::string;

#include <iostream>
using std::ostream;
using std::endl;

/**
 * A class for parsing and storing parameters for command line programs.
 * @author Juho Muhonen
 * @modified 6.10.2005: hasParameter(), commented all
 */
class ParameterParser {
private:
	/**
	 * Mapping of the parameters stored in this parser.
	 */
	map<string, string> parameters;
	
public:
	/**
	 * A constant value given for flag parameters
	 */
	static const string FLAG_PARAMETER;
	
	/**
	 * Creates an empty parameter parser.
	 */
	ParameterParser();

	/**
	 * Creates a parameter parser from main(int argc, char* argv[]) type of input.
	 */
	ParameterParser(int, char**);
	
	/**
	 * The destructor.
	 */
	~ParameterParser();

	/**
	 * Adds a parameter to this parser.
	 */
	void addParameter(string, string);
	
	/**
	 * Parses parameters from main(int argc, char* argv[]) type of input.
	 */
	void parseParameters(int, char**);
	
	/**
	 * Checks for a presence of a parameter.
	 */
	bool hasParameter(string);
	
	/**
	 * Checks if a parameter is only a flag(it has no actual value) parameter.
	 */
	bool isFlagParameter(string);
	
	/**
	 * Returns a parameter value.
	 */
	string getParameter(string);

	/**
	 * Prints this parser to a ostream.
	 */
	friend ostream& operator<<(ostream&, const ParameterParser&);
};

ostream& operator<<(ostream&, const ParameterParser&);

#endif
