/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_TEXTOUTPUTGENERATOR
#define FIRM_TEXTOUTPUTGENERATOR

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <iostream>
#include <fstream>
using std::ofstream;
using std::cout;
using std::ostream;

#include "OutputGenerator.h"

/**
 * A class for outputting association rules and itemsets in text format.
 * @author Juho Muhonen
 * @modification 9.11.2005: header line for rules and itemsets
 * @modification 7.11.2005: now supports precision
 * @modification 6.10.2005: commented all
 */
class TextOutputGenerator:public OutputGenerator {
private:
	/**
	 * The outputfile.
	 */
	ofstream* output_stream;
	
	/**
	 * The output stream, by default this is cout.
	 */
	ostream* out;
	
	/**
	 * The set constraints.
	 */
	vector<SetConstraint*> set_constraints;
	
	/**
	 * The rule constraints.
	 */
	vector<RuleConstraint*> rule_constraints;
	
	/**
	 * Outputs the header line for itemsets.
	 */
	void outputItemsetMeasures();
	
	/**
	 * Outputs the header line for association rules.
	 */
	void outputRuleMeasures();
public:
	/**
	 * Creates a new text output generator.
	 */
	TextOutputGenerator();

	/**
	 * The Destructor.
	 */
	~TextOutputGenerator();
	
	/**
	 * Sets a new name for the output file. If there is an existing output file,
	 * it is closed.
	 */
	void setOutputName(string);

	/**
	 * Adds a set constraint to this outputter.
	 */
	void addSetConstraint(SetConstraint*);

	/**
	 * Adds a rule constraint to this outputter.
	 */
	void addRuleConstraint(RuleConstraint*);
	
	/**
	 * Remove a constraint from this outputter.
	 */
	void removeConstraint(string);

	/**
	 * Output an itemsets.
	 */
	void outputSet(FITree*);
	
	/**
	 * Output all itemsets in a frequent itemset tree.
	 */
	void outputSets(FITree*);

	/**
	 * Output a single association rule.
	 */
	void outputRule(AssociationRule*);

	/**
	 * Output all possible association rules from a set of itemsets.
	 */
	void outputRules(FITree*);
	
	/**
	 * Finalizes this output generator after use. Namely flush and close output streams and delete all stored constraints.
	 */
	void finalize();
};
#endif
